from django.http.response import HttpResponse
from django.utils import translation

from .conf import (
    REDIRECT_EXCLUDE_PREFIXES,
    DEFAULT_REDIRECT_LANGUAGE,
    VISITED_SESSION_KEY
)
from .utils import (
    get_language_from_cookie,
    set_language_to_cookie_and_session,
    is_url_translatable,
    get_redirect_response_for_language
)

__all__ = (
    'language_redirect_middleware',
)


def language_redirect_middleware(get_response):
    """
    Middleware to redirect to default language
    """
    def middleware(request):
        response: 'HttpResponse' = get_response(request)
        current_language: str = translation.get_language()
        cookie_language: str = get_language_from_cookie(request)
        was_visited: str = (
            request.session.pop(
                VISITED_SESSION_KEY,
                request.META.get('HTTP_REFERER')
            )
        )

        is_skip: bool = (
            not is_url_translatable(
                url=request.path,
                language=DEFAULT_REDIRECT_LANGUAGE
            )
            or request.path.startswith(
                REDIRECT_EXCLUDE_PREFIXES
            )
            or response.status_code == 404
        )

        if is_skip:
            return response

        is_first_visit = (
            cookie_language is None
            and current_language != DEFAULT_REDIRECT_LANGUAGE
        )

        if is_first_visit:
            new_language = DEFAULT_REDIRECT_LANGUAGE
            response = (
                get_redirect_response_for_language(
                    language=new_language,
                    request=request
                )
            )

        elif cookie_language and current_language != cookie_language:
            if not bool(was_visited):
                new_language = cookie_language
            else:
                new_language = current_language

            response = (
                get_redirect_response_for_language(
                    language=new_language,
                    request=request
                )
            )
            request.session[VISITED_SESSION_KEY] = (
                request.build_absolute_uri(request.path)
            )

        else:
            new_language = current_language

        if cookie_language != new_language:
            set_language_to_cookie_and_session(
                request=request,
                response=response,
                language=new_language
            )

        return response

    return middleware



from urllib.parse import unquote

from django.http import HttpResponse, HttpResponseRedirect
from django.urls import translate_url
from django.utils.http import is_safe_url
from django.utils.translation import check_for_language

from .conf import REDIRECT_TO_QUERY_PARAMETER, LANGUAGE_QUERY_PARAMETER
from .utils import set_language_to_cookie_and_session

__all__ = (
    'set_language',
)


def set_language(request):
    """
    Redirect to a given URL while setting the chosen language in the session
    (if enabled) and in a cookie. The URL and the language code need to be
    specified in the request parameters.
    """
    redirect_to = request.GET.get(REDIRECT_TO_QUERY_PARAMETER)

    if (
            (redirect_to or not request.is_ajax())
            and not is_safe_url(
                url=redirect_to,
                allowed_hosts={request.get_host()},
                require_https=request.is_secure()
            )
    ):
        redirect_to = request.META.get('HTTP_REFERER')
        redirect_to = redirect_to and unquote(redirect_to)  # HTTP_REFERER may be encoded.

        if (
                not is_safe_url(
                    url=redirect_to,
                    allowed_hosts={request.get_host()},
                    require_https=request.is_secure()
                )
        ):
            redirect_to = '/'

    response = (
        HttpResponseRedirect(redirect_to)
        if redirect_to else HttpResponse(status=204)
    )

    lang_code = request.GET.get(LANGUAGE_QUERY_PARAMETER)

    if lang_code and check_for_language(lang_code):
        if redirect_to:
            redirect_to_trans = translate_url(redirect_to, lang_code)

            if redirect_to_trans != redirect_to:
                response = HttpResponseRedirect(redirect_to_trans)

        set_language_to_cookie_and_session(
            request=request,
            response=response,
            language=lang_code
        )

    return response



from typing import Tuple

from django.conf import settings

__all__ = (
    'REDIRECT_TO_QUERY_PARAMETER',
    'LANGUAGE_QUERY_PARAMETER',
    'REDIRECT_EXCLUDE_PREFIXES',
    'DEFAULT_REDIRECT_LANGUAGE',
    'VISITED_SESSION_KEY'
)


REDIRECT_TO_QUERY_PARAMETER: str = getattr(
    settings,
    'LANGUAGE_TOOLS_REDIRECT_TO_QUERY_PARAMETER',
    'redirect_to'
)


LANGUAGE_QUERY_PARAMETER: str = getattr(
    settings,
    'LANGUAGE_TOOLS_QUERY_PARAMETER',
    'language'
)

REDIRECT_EXCLUDE_PREFIXES: 'Tuple' = getattr(
    settings,
    'LANGUAGE_TOOLS_REDIRECT_EXCLUDE_PREFIXES',
    ()
)

DEFAULT_REDIRECT_LANGUAGE: str = getattr(
    settings,
    'LANGUAGE_TOOLS_DEFAULT_REDIRECT_LANGUAGE',
    settings.LANGUAGE_CODE
)

VISITED_SESSION_KEY: str = getattr(
    settings,
    'LANGUAGE_TOOLS_VISITED_SESSION_KEY',
    'language-tools:visited'
)



from setuptools import setup, find_packages

pkj_name = 'ok_language_tools'

setup(
    name='django-ok-language-tools',
    version='0.0.1',
    long_description_content_type='text/x-rst',
    packages=[pkj_name] + [pkj_name + '.' + x for x in find_packages(pkj_name)],
    include_package_data=True,
)
